package com.liangjian11.ums.utils;

import com.auth0.jwt.interfaces.RSAKeyProvider;
import com.liangjian11.ums.utils.content.Config;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.interfaces.DecodedJWT;

import java.io.UnsupportedEncodingException;
import java.util.Date;


public class JWTUtil {

    /**
     * 校验token是否正确
     *
     * @param token    密钥
     * @param username 用户名
     * @param secret   用户的密码
     * @return 是否正确
     */
    public static void verify(String token, String username, String secret) throws Exception {

        Algorithm algorithm = Algorithm.HMAC256(secret);

        JWTVerifier verifier = JWT.require(algorithm)
                .withClaim(Config.JWT_CLIENT_USERNAME, username)
                .withIssuer(Config.JWT_IUSER)
                .build();

        verifier.verify(token);
    }

    /**
     * 校验token是否正确
     *
     * @param token    密钥
     * @param username 用户名
     * @param secret   用户的密码
     * @return 是否正确
     */
    public static void verifyRSA(String token, String username, RSAKeyProvider secret) throws Exception {
        //其实按照规定只需要传递 publicKey 来校验即可，这可能是auth0 的缺点
        Algorithm algorithm = Algorithm.RSA256(secret.getPublicKeyById(""), secret.getPrivateKey());

        JWTVerifier verifier = JWT.require(algorithm)
                .withClaim(Config.JWT_CLIENT_USERNAME, username)
                .withIssuer(Config.JWT_IUSER)
                .build();

        verifier.verify(token);
    }

    /**
     * 校验token是否正确
     *
     * @param token    密钥
     * @return 是否正确
     */
    public static DecodedJWT decodeToken(String token){
            return JWT.decode(token);
    }

    /**
     * 获得token中自定义用户信息
     * @param token
     * @returntoken中包含的用户名
     */
    public static String getUsername(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim(Config.JWT_CLIENT_USERNAME).asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    /**
     * 获取token中的项目key
     * @param token
     * @return
     */
    public static String getApplicationKey(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim(Config.JWT_APPLICATION_KEY).asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    /**
     * 获取token中的角色key
     * @param token
     * @return
     */
    public static String getRoleKey(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim(Config.JWT_ROLE_KEY).asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    /**
     * 获取token中的部门code
     * @param token
     * @return
     */
    public static String getDeptCode(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim(Config.JWT_DEPT_CODE).asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    /**
     * 获得token中自定义用户信息
     * @param token
     * @returntoken中包含的用户名
     */
    public static String getUserValue(String token,String key) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim(key).asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    /**
     * 获取token中的grantType 授权类型
     * @param token
     * @return
     */
    public static String getGrantType(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim(Config.JWT_GRANT_TYPE).asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    /**
     * 获得token中自定义所有信息
     * @param token
     * @returntoken中包含的用户名
     */
    public static String getPayloadValue(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getPayload();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    public static String getPayloadValue(DecodedJWT jwt ,String token) {
        try {
            return jwt.getPayload();
        } catch (JWTDecodeException e) {
            return null;
        }
    }


    /**
     * 获取token中的有效期
     * @param token
     * @return
     */
    public static Date getExpiresAt(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getExpiresAt();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    public static Date getExpiresAt(DecodedJWT jwt,String token) {
        try {
            return jwt.getExpiresAt();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

    /**
     * 生成签名
     *
     * @param username   用户名
     * @param secret     用户的密码
     * @param activeTime 有效时间,毫秒
     * @param applicationKey 项目key
     * @return 加密的token
     */
    public static String sign(String username, String secret,
                              String applicationKey,
                              long activeTime) throws UnsupportedEncodingException {

        long currentTimes = System.currentTimeMillis();

        Date signdate = new Date(currentTimes);

        Date expireTime = new Date(currentTimes + activeTime);

        Algorithm algorithm = Algorithm.HMAC256(secret);

        return JWT.create()
                .withClaim(Config.JWT_CLIENT_USERNAME, username)
                .withClaim(Config.JWT_APPLICATION_KEY,applicationKey)
                .withIssuer(Config.JWT_IUSER)
                .withExpiresAt(expireTime)
                .withIssuedAt(signdate)
                .sign(algorithm);
    }

    /**
     * 生成签名
     *
     * @param username   用户名
     * @param secret     用户的密码
     * @param activeTime 有效时间,毫秒
     * @param applicationKey 项目key
     * @return 加密的token
     */
    public static String sign(String username, Object secret,
                              String applicationKey,
                              long activeTime,String secretType,
                              String grantType) throws UnsupportedEncodingException {

        long currentTimes = System.currentTimeMillis();

        Date signdate = new Date(currentTimes);

        Date expireTime = new Date(currentTimes + activeTime);

        Algorithm algorithm = null;
        if(secretType == null)
            algorithm = Algorithm.HMAC256((String)secret);
        else if(secretType == "RSA256")
            algorithm = Algorithm.RSA256((RSAKeyProvider)secret);
        return JWT.create()
                .withClaim(Config.JWT_CLIENT_USERNAME, username)
                .withClaim(Config.JWT_APPLICATION_KEY,applicationKey)
                .withIssuer(Config.JWT_IUSER)
                .withClaim(Config.JWT_GRANT_TYPE,grantType)
                .withExpiresAt(expireTime)
                .withIssuedAt(signdate)
                .sign(algorithm);
    }

    /**
     * 生成签名
     *
     * @param username   用户名
     * @param secret     用户的密码
     * @param activeTime 有效时间,毫秒
     * @param applicationKey 项目key
     * @param applicationSecret 项目密码
     * @return 加密的token
     */
    public static String sign(String username, String secret, String applicationKey, String applicationSecret,long activeTime) throws UnsupportedEncodingException {

        long currentTimes = System.currentTimeMillis();

        Date signdate = new Date(currentTimes);

        Date expireTime = new Date(currentTimes + activeTime);

        Algorithm algorithm = Algorithm.HMAC256(secret);

        return JWT.create()
                .withClaim(Config.JWT_CLIENT_USERNAME, username)
                .withClaim(Config.JWT_APPLICATION_KEY,applicationKey)
                .withClaim(Config.JWT_APPLICATION_SECRET,applicationSecret)
                .withIssuer(Config.JWT_IUSER)
                .withExpiresAt(expireTime)
                .withIssuedAt(signdate)
                .sign(algorithm);
    }

    /**
     * 生成签名
     *
     * @param username   用户名
     * @param secret     用户的密码
     * @param activeTime 有效时间,毫秒
     * @param applicationKey 项目key
     * @param roleKey 角色key
     * @param deptCode 部门编码
     * @param orgKey 机构key
     * @param grantType 授权类型
     * @param activeTime 有效时间
     * @return 加密的token
     */
    public static String sign(String username, String secret,
                              String applicationKey, String roleKey,
                              String deptCode, String orgKey,String extra,
                             long activeTime, String grantType) throws UnsupportedEncodingException {

        long currentTimes = System.currentTimeMillis();

        Date signdate = new Date(currentTimes);

        Date expireTime = new Date(currentTimes + activeTime);

        Algorithm algorithm = Algorithm.HMAC256(secret);

        return JWT.create()
                .withClaim(Config.JWT_CLIENT_USERNAME, username)
                .withClaim(Config.JWT_APPLICATION_KEY,applicationKey)
                .withClaim(Config.JWT_ROLE_KEY,roleKey)
                .withClaim(Config.JWT_DEPT_CODE,deptCode)
                .withClaim(Config.JWT_ORG_KEY,orgKey)
                .withClaim(Config.JWT_GRANT_TYPE,grantType)
                .withClaim(Config.JWT_Extra,extra)
                .withIssuer(Config.JWT_IUSER)
                .withExpiresAt(expireTime)
                .withIssuedAt(signdate)
                .sign(algorithm);
    }

    /**
     * 获取token中的角色key
     * @param jwt jwt token 参数
     * @param claim 身份信息
     * @return
     */
    public static String getClaim(DecodedJWT jwt,String claim) {
        try {
            return jwt.getClaim(claim).asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }

}
