package com.liangjian11.ums.web.shiro;

import com.auth0.jwt.interfaces.DecodedJWT;
import com.auth0.jwt.interfaces.RSAKeyProvider;
import com.liangjian11.ums.model.*;
import com.liangjian11.ums.service.*;
import com.liangjian11.ums.utils.CreateSecrteKey;
import com.liangjian11.ums.utils.JWTUtil;
import com.liangjian11.ums.utils.RSAUtil;
import com.liangjian11.ums.utils.content.Config;
import com.liangjian11.ums.exception.AuthException;
import org.apache.commons.lang3.StringUtils;
import org.apache.oltu.oauth2.common.message.types.GrantType;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.Permission;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.cache.Cache;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.util.*;


@Component
public class JWTRealm extends AuthorizingRealm {

    private static final Logger LOGGER = LoggerFactory.getLogger(JWTRealm.class);

    @Autowired
    @Qualifier("iUserService")
    private IUserService iUserService;

    @Autowired
    private ClientService clientService;

    @Autowired
    private IRoleService iRoleService;

    @Autowired
    private InterfaceResourceService iResourceService;

    @Autowired
    private IApplicationService iApplicationService;

    @Value("${account.security.privatekey}")
    private String privateKey;

    /**
     * 大坑！，必须重写此方法，不然Shiro会报错
     */
    @Override
    public boolean supports(AuthenticationToken token) {
        return token instanceof JWTToken;
    }

    /**
     * 只有当需要检测用户权限的时候才会调用此方法
     */
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principals) {
        String jwtToken = principals.toString();
        DecodedJWT decodedJWT = JWTUtil.decodeToken(jwtToken);
        String username = JWTUtil.getUsername(jwtToken);
        String applicationKey = JWTUtil.getApplicationKey(jwtToken);
        String grantType = JWTUtil.getClaim(decodedJWT,Config.JWT_GRANT_TYPE);


        Account user = this.iUserService.getUserByUserName(username);
        List<Role> roles = this.iRoleService.getRolesByUserAndAppKey(user, applicationKey);

        if( grantType != null && grantType.equals(Config.ClientCredentials) ){
            user = new Account();
            user.setLoginId(username);
        }

        List<InterfaceResource> resources;

        if (this.iRoleService.hasRole(roles, Config.SUPER_ADMIN_ROLE)) {

            /**
             * 若为超级管理员，加载该项目所有权限
             */
            resources = this.iResourceService.getResourcesByAppKey(applicationKey,-1);

        } else {

            resources = this.iResourceService.getResourcesByUserAndAppKey(user, applicationKey,-1,grantType);

        }

        return this.buildSimpleAuthorizationInfo(roles, resources);
    }


    protected boolean isPermitted(Permission permission, AuthorizationInfo info) {
        Collection<Permission> perms = this.getPermissions(info);
        if (perms != null && !perms.isEmpty()) {
            Iterator var4 = perms.iterator();

            while(var4.hasNext()) {
                Permission perm = (Permission)var4.next();
                if(perm.toString().contains("*")) {
                    String perstr = perm.toString().replace("*","");
                    if(permission.toString().startsWith(perstr.toString()))
                        return true;

                }else  if (perm.implies(permission)) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * 根据身份获取存储redis的key
     */
    @Override
    protected Object getAuthorizationCacheKey(PrincipalCollection principals){
//        return OAuthServiceImpl.ACSTOKEN_PRE+principals.toString();
        return principals;
    }

    /**
     * 根据身份获取存储的授权信息 先不经过缓存 直接从数据库读取
     */
    @Override
    protected AuthorizationInfo getAuthorizationInfo(PrincipalCollection principals) {
        if (principals == null) {
            return null;
        } else {
            AuthorizationInfo info = null;
//            if (log.isTraceEnabled()) {
//                log.trace("Retrieving AuthorizationInfo for principals [" + principals + "]");
//            }

//            Cache<Object, AuthorizationInfo> cache = this.getAvailableAuthorizationCache();
            Cache<Object, AuthorizationInfo> cache = null;
            Object key;
            if (cache != null) {
//                if (log.isTraceEnabled()) {
//                    log.trace("Attempting to retrieve the AuthorizationInfo from cache.");
//                }

                key = this.getAuthorizationCacheKey(principals);
                info = (AuthorizationInfo)cache.get(key);
//                if (log.isTraceEnabled()) {
//                    if (info == null) {
//                        log.trace("No AuthorizationInfo found in cache for principals [" + principals + "]");
//                    } else {
//                        log.trace("AuthorizationInfo found in cache for principals [" + principals + "]");
//                    }
//                }
            }

            if (info == null) {
                info = this.doGetAuthorizationInfo(principals);
                if (info != null && cache != null) {
//                    if (log.isTraceEnabled()) {
//                        log.trace("Caching authorization info for principals: [" + principals + "].");
//                    }

                    key = this.getAuthorizationCacheKey(principals);
                    cache.put(key, info);
                }
            }

            return info;
        }
    }

    /**
     * 默认使用此方法进行用户名正确与否验证，错误抛出异常即可。
     */
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken auth) throws AuthException {
        String jwtToken = (String) auth.getCredentials();

        String grantType = JWTUtil.getGrantType(jwtToken);

        // 解密获得username，用于和数据库进行对比
        String username = JWTUtil.getUsername(jwtToken);
        String applicationKey = JWTUtil.getApplicationKey(jwtToken);
        if (StringUtils.isBlank(username) || StringUtils.isBlank(applicationKey)) {
            throw new AuthException("Token is invalid!");
        }

        Application application = this.iApplicationService.getApplicationByAppkey(applicationKey);
        if (Objects.isNull(application)) {
            throw new AuthException("Application " + applicationKey + " doesn't exist!");
        }

        String password = null;
        grantType = grantType.replace("clientCredentials","client_credentials");
        grantType = grantType.replace("authorizationCode","authorization_code");
        if(grantType != null) {
            if (grantType.equals(GrantType.PASSWORD.toString()) || grantType.equals(GrantType.AUTHORIZATION_CODE.toString())) {

                Account user = this.iUserService.getUserByUserName(username);
                if (user == null) {
                    throw new AuthException("User doesn't exist!");
                }

                try {
                    password = RSAUtil.decrypt(user.getPassword(),privateKey);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                try {
                    JWTUtil.verify(jwtToken, username, password);
                } catch (Exception e) {
                    LOGGER.error("Method doGetAuthenticationInfo occur an unknown error!", e);
                    throw new AuthException(e.getMessage());
                }
            } else if (grantType.equals(GrantType.CLIENT_CREDENTIALS.toString())) {

                Client client = this.clientService.findByClientId(username);
                if (client == null) {
                    throw new AuthException("Client doesn't exist!");
                }
                final String passwordPub = client.getClientSecret();
                String clientPrivSecret = client.getClientPrivSecret();

                RSAKeyProvider keyProvider = new RSAKeyProvider(){
                    public RSAPublicKey getPublicKeyById(String var1){
                        try {
                            return (RSAPublicKey) CreateSecrteKey.getPublicKey(passwordPub);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                        return null;
                    }

                    public RSAPrivateKey getPrivateKey(){
                        try {
                            return (RSAPrivateKey) CreateSecrteKey.getPrivateKey(clientPrivSecret);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                        return null;
                    }

                    public String getPrivateKeyId(){
                        return "";
                    }
                };

                try {
                    JWTUtil.verifyRSA(jwtToken, username, keyProvider);
                } catch (Exception e) {
                    LOGGER.error("Method doGetAuthenticationInfo occur an unknown error!", e);
                    throw new AuthException(e.getMessage());
                }
            }
        }


        return new SimpleAuthenticationInfo(jwtToken, jwtToken, "jwtRealm");
    }

    private SimpleAuthorizationInfo buildSimpleAuthorizationInfo(List<Role> roles, List<InterfaceResource> resources) {

        SimpleAuthorizationInfo simpleAuthorizationInfo = new SimpleAuthorizationInfo();

        if (Objects.nonNull(roles) && roles.size() > 0) {
            Set<String> roleSets = new LinkedHashSet<>();
            for (Role role : roles) {
                roleSets.add(role.getRoleKey());
            }
            simpleAuthorizationInfo.addRoles(roleSets);
        }

        if (Objects.nonNull(resources) && resources.size() > 0) {
            Set<String> permissionSets = new LinkedHashSet<>();
            for (InterfaceResource resource : resources) {
                if(resource.getLinkUrl()!=null &&!resource.getLinkUrl().isEmpty() )//资源url不能为null 或空字符串
                    permissionSets.add(resource.getLinkUrl());
            }
            simpleAuthorizationInfo.addStringPermissions(permissionSets);
        }

        return simpleAuthorizationInfo;
    }

}
